# frozen_string_literal: true

#
# Cookbook Name:: samhain
# Library:: resource/samhain_config
#
# Copyright:: 2015, Tyler Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require 'chef/mixin/deep_merge'
require 'chef/resource'
require_relative '../helpers/config'

class Chef
  class Resource
    # A Chef resource for a Samhain configuration.
    class SamhainConfig < Resource
      provides :samhain_config

      default_action :create

      #
      # Set a default name since it's never used.
      #
      property :name, String, default: 'default'

      #
      # Property for an input config hash.
      #
      property :config, Hash, default: {}

      #
      # Generate a config file based on the input config hash.
      # At least on Ubuntu, Samhain is not compiled with any trusted users and
      # will fail to start because of /var/log being group-writable, so always
      # ensure that the relevant user(s) get(s) trusted in those cases.
      #
      action :create do
        file '/etc/samhain/samhainrc' do
          owner 'root'
          group 'root'
          mode '0644'
          content(
            lazy do
              if SamhainCookbook::Helpers::Config.group_writable?('/var/log')
                trusted = new_resource.config.to_h.dig('Misc', 'TrustedUser')
                us = trusted.nil? ? [] : trusted.split(',')
                us += SamhainCookbook::Helpers::Config
                  .users_with_group_write_access_for('/var/log')

                conf = Chef::Mixin::DeepMerge
                  .merge(new_resource.config.to_h,
                         'Misc' => { 'TrustedUser' => us.uniq.join(',') })

                SamhainCookbook::Helpers::Config.build_config(conf)
              else
                SamhainCookbook::Helpers::Config.build_config(new_resource.config)
              end
            end
          )
        end
      end

      #
      # Delete the samhainrc config file.
      #
      action :remove do
        file('/etc/samhain/samhainrc') { action :delete }
      end
    end
  end
end
